/* MPI Gather Offset Example */
#include <stdio.h>
#include "mpi.h"

#define ARRAYSIZE 64

int A[ARRAYSIZE];
int B[ARRAYSIZE];


int main(int argc, char **argv) {

	int num_ranks, rank;
	int result,i,length;

	/* Initialize MPI */
	result = MPI_Init(&argc,&argv);
	if (result != MPI_SUCCESS) {
		printf ("Error starting MPI program!.\n");
		MPI_Abort(MPI_COMM_WORLD, result);
	}

	/* Get number of tasks and our process number (rank) */
	MPI_Comm_size(MPI_COMM_WORLD,&num_ranks);
	MPI_Comm_rank(MPI_COMM_WORLD,&rank);

	printf("R%d: Number of ranks= %d My rank= %d\n",
		rank,num_ranks,rank);

	/* Initialize Arrays */
	printf("R%d: Initializing array to i\n",rank);
	for(i=0;i<ARRAYSIZE;i++) {
		A[i]=i;
	}

	MPI_Barrier(MPI_COMM_WORLD);

	/* first attempt, plain */
	if (rank==0) {
		printf("First attempt, plain (send buffer=A)\n");
	}

	length=ARRAYSIZE/num_ranks;

	MPI_Gather(A,		/* send buffer */
		length,		/* send count */
		MPI_INT,	/* type */
		B,		/* receive buffer */
		length,		/* receive count */
		MPI_INT,	/* type */
		0,		/* root source */
		MPI_COMM_WORLD);

	if (rank==0) {
		printf("R%d: B[]=\n",rank);
		for(i=0;i<ARRAYSIZE;i++) {
			if (i%16==0) printf("\t");
			printf("%d ",B[i]);
			if (i%16==15) printf("\n");
		}
	}

	MPI_Barrier(MPI_COMM_WORLD);

	/* second attempt, offset */
	if (rank==0) {
		printf("Second attempt, offset (send_buffer=&A[rank*length]\n");
	}

	length=ARRAYSIZE/num_ranks;

	MPI_Gather(
		&A[rank*length],
		//&A[i*length],		/* send buffer (segfaults)*/
		length,		/* send count */
		MPI_INT,	/* type */
		B,		/* receive buffer */
		length,		/* receive count */
		MPI_INT,	/* type */
		0,		/* root source */
		MPI_COMM_WORLD);

	if (rank==0) {
		printf("R%d: B[]=\n",rank);
		for(i=0;i<ARRAYSIZE;i++) {
			if (i%16==0) printf("\t");
			printf("%d ",B[i]);
			if (i%16==15) printf("\n");
		}
	}


	MPI_Finalize();
}
