/* MPI GatherV Example */

/* see if we can make it work for MPI_IN_PLACE */

/* put MPI_IN_PLACE in sendbuf, sendcount/type ignored */

#include <stdio.h>
#include <stdlib.h>

#include <mpi.h>

#define ARRAYSIZE 64

int A[ARRAYSIZE];
//int B[ARRAYSIZE];


int main(int argc, char **argv) {

	int num_ranks, rank;
	int result,i;
	int *offsets,*counts;

	/* Initialize MPI */
	result = MPI_Init(&argc,&argv);
	if (result != MPI_SUCCESS) {
		printf ("Error starting MPI program!.\n");
		MPI_Abort(MPI_COMM_WORLD, result);
	}

	/* Get number of tasks and our process number (rank) */
	MPI_Comm_size(MPI_COMM_WORLD,&num_ranks);
	MPI_Comm_rank(MPI_COMM_WORLD,&rank);

	printf("R%d: Number of ranks= %d My rank= %d\n",
		rank,num_ranks,rank);

	/* Initialize Arrays */
	printf("R%d: Initializing array to %d\n",rank,rank);
	for(i=0;i<ARRAYSIZE;i++) {
		A[i]=rank;
	}


	offsets=calloc(num_ranks,sizeof(int));
	counts=calloc(num_ranks,sizeof(int));

	for(i=0;i<num_ranks;i++) {
		offsets[i]=i*(ARRAYSIZE/num_ranks);
		counts[i]=ARRAYSIZE/num_ranks;
		if (i==num_ranks-1) counts[i]+=(ARRAYSIZE%num_ranks);
		printf("counts[%d]=%d\n",i,counts[i]);
	}

	if (rank==0) {
		MPI_Gatherv(MPI_IN_PLACE,/* send buffer */
			counts[rank],	/* send count */
			MPI_INT,	/* type */
			A,		/* receive buffer */
			counts,		/* receive count */
			offsets,	/* receive offsets */
			MPI_INT,	/* type */
			0,		/* root source */
			MPI_COMM_WORLD);
	}
	else {
		MPI_Gatherv(A,	/* send buffer */
			counts[rank],	/* send count */
			MPI_INT,	/* type */
			NULL,		/* receive buffer */
			counts,		/* receive count */
			offsets,	/* receive offsets */
			MPI_INT,	/* type */
			0,		/* root source */
			MPI_COMM_WORLD);
	}

	if (rank==0) {
		printf("R%d: A[]=\n",rank);
		for(i=0;i<ARRAYSIZE;i++) {
			if (i%16==0) printf("\t");
			printf("%d ",A[i]);
			if (i%16==15) printf("\n");
		}
	}

	MPI_Finalize();
}
